"use strict";

this.name = "Solar Harvester AI";
this.version = "0.1.0";
this.author = "ocz";
this.copyright = "ocz 2016";
this.license = "cc by-nc-sa 4.0";

this.entityDestroyed = function() {
	if (this._solarHarvesterTorusSpeedTimer) {
		if (this._solarHarvesterTorusSpeedTimer.isRunning) this._solarHarvesterTorusSpeedTimer.stop();
		delete this._solarHarvesterTorusSpeedTimer;
		delete this;
	}
}

this.aiStarted = function() {
	
	var ai = new worldScripts["oolite-libPriorityAI"].PriorityAIController(this.ship);
	ai.setCommunicationsRole("scavenger");
	ai.setParameter("_solarHarvesterReturnHome", false);	
	
	ai._solarHarvesterTorusDrive = function() {
		var timer = this.ship.AIScript._solarHarvesterTorusSpeedTimer;
		if (this.ship.checkScanner().length !== 0 || this.ship.position.distanceTo(system.sun.position) < system.sun.radius * 1.48) {
			if (timer.isRunning) this.ship.velocity = this.ship.heading.multiply(this.ship.maxSpeed);// Drop out of Torus Speed to maxSpeed. Deceleration to set speed happens naturally afterwards.
			timer.stop();
		}
		else this.ship.velocity = this.ship.heading.multiply(this.ship.maxSpeed * 32); // Accelerate to or refresh Torus Speed (32*maxSpeed).
		if (!this.ship && timer.isRunning) timer.stop();
	}

	// Engage Torusdrive, check every 0.25 sec if there is something masslocking (excluding Planets/Suns, including asteroids/loot) and stop or maintain Torus Speed (refresh velocity).
	this._solarHarvesterTorusSpeedTimer = new Timer(ai, ai._solarHarvesterTorusDrive ,-1 ,0.25); 
	
	ai._solarHarvesterReturnHome = function() {
		if (ai.getParameter("_solarHarvesterReturnHome")) return ai.getParameter("_solarHarvesterReturnHome");
		return false;
	}
	
	ai._setSolarHarvesterReturnHome = function(bool) {
		return ai.setParameter("_solarHarvesterReturnHome", bool);
	}
	
	
	ai._solarHarvesterConditionCargoHoldFull = function() {
		if (this.ship.cargoSpaceAvailable !== 0) {
			return false;
		}
		return true;
	}

	ai._solarHarvesterConditionReadyToHarvest = function () {
		if (this.ship.cargoSpaceAvailable === 0) return false;
		if (this.distance(system.sun) < system.sun.radius * 1.5) return true;
		return false;
	}

	ai._solarHarvesterConditionTooCloseToSun = function () {
		if (this.distance(system.sun) < system.sun.radius * 1.45) return true;
		return false;
	}

	ai._solarHarvesterConditionTooHotAndNotAlerted = function () {
		if (ai._solarHarvesterReturnHome() === true) return false;
		if (this.ship.temperature > 0.55) return true;
		return false;
	}

	ai._solarHarvesterConditionReturnHome = function() {
		if (ai._solarHarvesterReturnHome() === true || ai._solarHarvesterConditionCargoHoldFull() === true) return true;
		return false;
	}

	ai._solarHarvesterConditionDockHome = function() {
		if (ai._solarHarvesterConditionHomeStationNearby() === true && ai._solarHarvesterConditionReturnHome() === true) return true;
		return false;
	}

	
	ai._solarHarvesterBehaviourGotoHarvestOrbitAboveSun = function() {
		this.ship.destination = system.sun.position;
		this.ship.desiredRange = system.sun.radius * 1.48;
		this.ship.desiredSpeed = this.ship.maxSpeed;
		ai.behaviourApproachDestination();
	}
	
	
	ai.setPriorities([
		// Fight 
		{
			condition: ai.conditionInCombat,
			behaviour: function() {
					ai._setSolarHarvesterReturnHome(true);
					ai.behaviourFleeCombat();
				},
			reconsider: 5
		},
		
		// Mugged? 
		{
			condition: ai.conditionPiratesCanBePaidOff,
			behaviour: function() {
					ai._setSolarHarvesterReturnHome(true);
					ai.behaviourPayOffPirates();
				},
			reconsider: 10
		},
		
		// Too hot for you? 
		{
			condition: ai._solarHarvesterConditionTooHotAndNotAlerted,
			configuration: ai.configurationSetDestinationToHomeStation,
			behaviour: function() {
					ai._setSolarHarvesterReturnHome(true);
					ai.behaviourApproachDestination();
					this.ship.desiredSpeed = this.ship.maxSpeed;
				},
			reconsider: 10
		},
		
		// Don't hang around near hostile stations 
		{
			condition: ai.conditionHostileStationNearby,
			configuration: ai.configurationSetDestinationToNearestHostileStation,
			behaviour: ai.behaviourLeaveVicinityOfDestination,
			reconsider: 20
		},
		
		// Is there any loot? 
		{
			preconfiguration: ai.configurationCheckScanner,
			condition: function() {
					// If the temperature is alright, the cargohold has space left and there is loot, go for it.
					if (this.ship.temperature < 0.55 && !ai._solarHarvesterConditionCargoHoldFull && ai.conditionScannerContainsSalvageForMe() ) return true;
					return false;
				},
			configuration: ai.configurationAcquireScannedTarget,
			behaviour: ai.behaviourCollectSalvage,
			reconsider: 20
		},
		
		// Dock at home.
		{
			condition: function() {
					// If you are on your way back and the station is nearby, start docking manuver.
					if (ai.conditionHomeStationNearby() === true && ai._solarHarvesterConditionReturnHome() === true) return true;
					return false;
				},
			configuration: ai.configurationSetHomeStationForDocking,
			behaviour: ai.behaviourDockWithStation,
			reconsider: 30
		},
		
		// Cargo hold full? Attacked? Bad sunburn? Go home.
		{
			condition: function() {
					if (ai._solarHarvesterConditionReturnHome() === true && ai.conditionHomeStationExists() === true) return true;
					return false;
				},
			configuration: ai.configurationSetDestinationToHomeStation,
			behaviour: function() {
					ai.behaviourApproachDestination();
					this.ship.desiredSpeed = this.ship.maxSpeed;				
					this.ship.AIScript._solarHarvesterTorusSpeedTimer.start();
				},
			reconsider: 20
		},
		
		// Or at least return to somewhere 
		{
			condition: function() {
					// If you are on your way back and any friendly station is nearby (If you want to return and made it this far in the script your home station is no more), start docking manuver.
					if (ai.conditionFriendlyStationNearby() === true && ai._solarHarvesterConditionReturnHome() === true) return true;
					return false;
				},
			configuration: ai.configurationSetNearbyFriendlyStationForDocking,
			behaviour: ai.behaviourDockWithStation,
			reconsider: 30
		},
		{
			condition: function() {
					if (ai.conditionFriendlyStationExists() === true && ai._solarHarvesterConditionReturnHome() === true) return true;
					return false;
				},
			configuration: ai.configurationSetDestinationToNearestFriendlyStation,
			behaviour: function() {
					ai.behaviourApproachDestination();
					this.ship.desiredSpeed = this.ship.maxSpeed;				
					this.ship.AIScript._solarHarvesterTorusSpeedTimer.start();
				},
			reconsider: 20
		},
		
		// Too close to the Sun? 
		{
			condition: ai._solarHarvesterConditionTooCloseToSun,
			behaviour: function() {
					ai._solarHarvesterBehaviourGotoHarvestOrbitAboveSun();
					this.ship.adjustCargo("coronal_plasma",1); // 1t Coronal Plasma every executuion (reconsider: 10 -> every 10s)
				},
			reconsider: 10
		},
		
		// Plasma harvest
		{
			condition: ai._solarHarvesterConditionReadyToHarvest,
			configuration: ai.configurationSetDestinationToSunskimEnd,
			behaviour: function() {
					ai.behaviourApproachDestination();
					this.ship.adjustCargo("coronal_plasma",1); // 1t every executuion (reconsider: 10 -> every 10s)
				},
			reconsider: 10
		},
		
		// Go to sun
		{
			condition: !ai._solarHarvesterConditionReadyToHarvest,
			behaviour: function () {
					ai._solarHarvesterBehaviourGotoHarvestOrbitAboveSun();
					this.ship.AIScript._solarHarvesterTorusSpeedTimer.start();
				},
			reconsider: 20
		} 
		
	].concat(ai.templateWitchspaceJumpAnywhere()));
}
